#ifndef POKEPLATINUM_OV55_0225632C_H
#define POKEPLATINUM_OV55_0225632C_H

#include "struct_decls/struct_02018340_decl.h"
#include "overlay055/struct_ov55_0225632C_1.h"
#include "overlay055/struct_ov55_0225632C_decl.h"

BOOL ov55_0225632C(UnkStruct_ov55_0225632C ** param0, const UnkStruct_ov55_0225632C_1 * param1, UnkStruct_02018340 * param2);
void ov55_02256360(UnkStruct_ov55_0225632C * param0);
void ov55_02256378(UnkStruct_ov55_0225632C * param0, u32 param1);
BOOL ov55_0225639C(UnkStruct_ov55_0225632C * param0, u32 param1);
BOOL ov55_022563A8(UnkStruct_ov55_0225632C * param0);

#endif // POKEPLATINUM_OV55_0225632C_H
